package io.annaclemens.xivchat

import android.content.Context
import androidx.room.Room
import androidx.room.RoomDatabase
import androidx.room.migration.Migration
import androidx.sqlite.db.SupportSQLiteDatabase
import io.annaclemens.xivchat.model.SavedServer
import io.annaclemens.xivchat.model.SavedServerDao
import io.annaclemens.xivchat.model.Tab
import io.annaclemens.xivchat.model.TabDao

@androidx.room.Database(entities = [SavedServer::class, Tab::class], version = 4, exportSchema = false)
abstract class Database : RoomDatabase() {
    abstract fun serverDao(): SavedServerDao
    abstract fun tabDao(): TabDao

    companion object {
        @Volatile
        private var INSTANCE: Database? = null

        fun getDatabase(context: Context): Database {
            val tempInstance = this.INSTANCE
            if (tempInstance != null) {
                return tempInstance
            }
            synchronized(this) {
                val instance = Room.databaseBuilder(context.applicationContext, Database::class.java, "xivchat")
                    .fallbackToDestructiveMigrationFrom(1)
                    .addMigrations(
                        AddTabsMigration(),
                        AddRelayMigration(),
                    )
                    .build()
                this.INSTANCE = instance
                return instance
            }
        }
    }

    private class AddTabsMigration : Migration(2, 3) {
        override fun migrate(database: SupportSQLiteDatabase) {
            database.execSQL("create table tabs (id integer primary key autoincrement not null, name text not null, filters text not null);")
        }
    }

    private class AddRelayMigration : Migration(3, 4) {
        override fun migrate(database: SupportSQLiteDatabase) {
            database.execSQL("alter table servers rename to servers_old")
            database.execSQL("create table servers (id integer primary key autoincrement not null, name text not null, host text default null, port integer default null, auth text default null, target text default null)")
            database.execSQL("insert into servers (id, name, host, port) select id, name, host, port from servers_old")
            database.execSQL("drop table servers_old")
        }
    }
}
