package io.annaclemens.xivchat

import android.content.Context
import android.graphics.Canvas
import android.graphics.drawable.ShapeDrawable
import android.graphics.drawable.shapes.RectShape
import android.util.AttributeSet
import android.view.View
import io.annaclemens.xivchat.util.toColours

class KeyVisualisation(context: Context, attrs: AttributeSet) : View(context, attrs) {
    private val drawables: MutableList<ShapeDrawable> = ArrayList()

    override fun onMeasure(widthMeasureSpec: Int, heightMeasureSpec: Int) {
        val heightPx = 16 * this.context.resources.displayMetrics.density
        val height = MeasureSpec.makeMeasureSpec(heightPx.toInt(), MeasureSpec.EXACTLY)
        super.onMeasure(widthMeasureSpec, height)
    }

    fun generateDrawables(bytes: ByteArray) {
        this.drawables.clear()
        val colours = bytes.toColours()
        for (colour in colours) {
            val drawable = ShapeDrawable(RectShape()).apply {
                this.paint.color = colour.toInt()
            }
            this.drawables.add(drawable)
        }
    }

    override fun onDraw(canvas: Canvas) {
        val width = this.width
        val height = this.height

        if (this.drawables.isEmpty()) {
            return
        }

        val chunkWidth = width / this.drawables.size

        for (drawable in this.drawables.withIndex()) {
            drawable.value.setBounds(drawable.index * chunkWidth, 0, (drawable.index + 1) * chunkWidth, height)
            drawable.value.draw(canvas)
        }
    }
}
