package io.annaclemens.xivchat.model

@OptIn(ExperimentalUnsignedTypes::class)
class ChatCode(val code: UInt) {
    companion object {
        private val CLEAR_7: UInt = (0u.inv() shl 7).inv()
    }

    fun source(): ChatSource? {
        if (this.code <= Companion.CLEAR_7) {
            return null
        }

        return this.sourceFrom(11)
    }

    fun target(): ChatSource? {
        if (this.code <= Companion.CLEAR_7) {
            return null
        }

        return this.sourceFrom(7)
    }

    private fun sourceFrom(shift: Int): ChatSource? {
        val shifted = (this.code shr shift).toInt() and 0xF
        val code = (1 shl shifted).toUShort()
        return ChatSource.fromCode(code)
    }

    fun type(): ChatType? {
        val type = this.code and Companion.CLEAR_7
        return ChatType.fromCode(type)
    }
}

@OptIn(ExperimentalUnsignedTypes::class)
enum class ChatSource(val code: UShort) {
    Self(2u),
    PartyMember(4u),
    AllianceMember(8u),
    Other(16u),
    EngagedEnemy(32u),
    UnengagedEnemy(64u),
    FriendlyNpc(128u),
    SelfPet(256u),
    PartyPet(512u),
    AlliancePet(1024u),
    OtherPet(2048u);

    companion object {
        val OTHERS = listOf(
            PartyMember,
            AllianceMember,
            Other,
            EngagedEnemy,
            UnengagedEnemy,
            FriendlyNpc,
            PartyPet,
            AlliancePet,
            OtherPet,
        )

        private val map = values().associateBy(ChatSource::code)
        fun fromCode(code: UShort) = this.map[code]
    }
}

@OptIn(ExperimentalUnsignedTypes::class)
enum class ChatType(val code: UInt) {
    AppSystem(0u),

    Debug(1u),
    Urgent(2u),
    Notice(3u),
    Say(10u),
    Shout(11u),
    TellOutgoing(12u),
    TellIncoming(13u),
    Party(14u),
    Alliance(15u),
    Linkshell1(16u),
    Linkshell2(17u),
    Linkshell3(18u),
    Linkshell4(19u),
    Linkshell5(20u),
    Linkshell6(21u),
    Linkshell7(22u),
    Linkshell8(23u),
    FreeCompany(24u),
    NoviceNetwork(27u),
    CustomEmote(28u),
    StandardEmote(29u),
    Yell(30u),
    // 31 - also party?
    CrossParty(32u),
    PvpTeam(36u),
    CrossLinkshell1(37u),
    Damage(41u),
    Miss(42u),
    Action(43u),
    Item(44u),
    Healing(45u),
    GainBuff(46u),
    GainDebuff(47u),
    LoseBuff(48u),
    LoseDebuff(49u),
    Alarm(55u),
    Echo(56u),
    System(57u),
    BattleSystem(58u),
    GatheringSystem(59u),
    Error(60u),
    NpcDialogue(61u),
    LootNotice(62u),
    Progress(64u),
    LootRoll(65u),
    Crafting(66u),
    Gathering(67u),
    NpcAnnouncement(68u),
    FreeCompanyAnnouncement(69u),
    FreeCompanyLoginLogout(70u),
    RetainerSale(71u),
    PeriodicRecruitmentNotification(72u),
    Sign(73u),
    RandomNumber(74u),
    NoviceNetworkSystem(75u),
    Orchestrion(76u),
    PvpTeamAnnouncement(77u),
    PvpTeamLoginLogout(78u),
    MessageBook(79u),
    GmTell(80u),
    GmSay(81u),
    GmShout(82u),
    GmYell(83u),
    GmParty(84u),
    GmFreeCompany(85u),
    GmLinkshell1(86u),
    GmLinkshell2(87u),
    GmLinkshell3(88u),
    GmLinkshell4(89u),
    GmLinkshell5(90u),
    GmLinkshell6(91u),
    GmLinkshell7(92u),
    GmLinkshell8(93u),
    GmNoviceNetwork(94u),
    CrossLinkshell2(101u),
    CrossLinkshell3(102u),
    CrossLinkshell4(103u),
    CrossLinkshell5(104u),
    CrossLinkshell6(105u),
    CrossLinkshell7(106u),
    CrossLinkshell8(107u);

    companion object {
        private val map = values().associateBy(ChatType::code)
        fun fromCode(code: UInt) = this.map[code]
    }

    fun isBattle() = when (this) {
        Damage, Miss, Action, Item, Healing, GainBuff, LoseBuff, GainDebuff, LoseDebuff, BattleSystem -> true
        else -> false
    }
}
