package io.annaclemens.xivchat.model.message

import io.annaclemens.xivchat.R

enum class PlayerListType(val code: Byte) {
    Party(1),
    Friend(2),
    Linkshell(3),
    CrossLinkshell(4),
    Targeting(5);

    companion object {
        private val map = values().associateBy(PlayerListType::code)
        fun fromCode(code: Byte) = this.map[code]
    }
}

enum class InputChannel(val code: Byte) {
    Tell(0),
    Say(1),
    Party(2),
    Alliance(3),
    Yell(4),
    Shout(5),
    FreeCompany(6),
    PvpTeam(7),
    NoviceNetwork(8),
    CrossLinkshell1(9),
    CrossLinkshell2(10),
    CrossLinkshell3(11),
    CrossLinkshell4(12),
    CrossLinkshell5(13),
    CrossLinkshell6(14),
    CrossLinkshell7(15),
    CrossLinkshell8(16),
    Linkshell1(19),
    Linkshell2(20),
    Linkshell3(21),
    Linkshell4(22),
    Linkshell5(23),
    Linkshell6(24),
    Linkshell7(25),
    Linkshell8(26);

    companion object {
        private val map = values().associateBy(InputChannel::code)
        fun fromCode(code: Byte) = this.map[code]
    }
}

enum class OnlineStatus(val code: Byte, val stringRes: Int) {
    GameQa(1, R.string.status_game_qa),
    GameMaster1(2, R.string.status_game_master),
    GameMaster2(3, R.string.status_game_master),
    EventParticipant(4, R.string.status_event_participant),
    Disconnected(5, R.string.status_disconnected),
    WaitingForFriendListApproval(6, R.string.status_waiting_for_friend_list_approval),
    WaitingForLinkshellApproval(7, R.string.status_waiting_for_linkshell_approval),
    WaitingForFreeCompanyApproval(8, R.string.status_waiting_for_free_company_approval),
    NotFound(9, R.string.status_not_found),
    Offline(10, R.string.status_offline),
    BattleMentor(11, R.string.status_battle_mentor),
    Busy(12, R.string.status_busy),
    Pvp(13, R.string.status_pvp),
    PlayingTripleTriad(14, R.string.status_playing_triple_triad),
    ViewingCutscene(15, R.string.status_viewing_cutscene),
    UsingAChocoboPorter(16, R.string.status_using_a_chocobo_porter),
    AwayFromKeyboard(17, R.string.status_away_from_keyboard),
    CameraMode(18, R.string.status_camera_mode),
    LookingForRepairs(19, R.string.status_looking_for_repairs),
    LookingToRepair(20, R.string.status_looking_to_repair),
    LookingToMeldMateria(21, R.string.status_looking_to_meld_materia),
    RolePlaying(22, R.string.status_role_playing),
    LookingForParty(23, R.string.status_looking_for_party),
    SwordForHire(24, R.string.status_sword_for_hire),
    WaitingForDutyFinder(25, R.string.status_waiting_for_duty_finder),
    RecruitingPartyMembers(26, R.string.status_recruiting_party_members),
    Mentor(27, R.string.status_mentor),
    PveMentor(28, R.string.status_pve_mentor),
    TradeMentor(29, R.string.status_trade_mentor),
    PvpMentor(30, R.string.status_pvp_mentor),
    Returner(31, R.string.status_returner),
    NewAdventurer(32, R.string.status_new_adventurer),
    AllianceLeader(33, R.string.status_alliance_leader),
    AlliancePartyLeader(34, R.string.status_alliance_party_leader),
    AlliancePartyMember(35, R.string.status_alliance_party_member),
    PartyLeader(36, R.string.status_party_leader),
    PartyMember(37, R.string.status_party_member),
    PartyLeaderCrossWorld(38, R.string.status_party_leader_cross_world),
    PartyMemberCrossWorld(39, R.string.status_party_member_cross_world),
    AnotherWorld(40, R.string.status_another_world),
    SharingDuty(41, R.string.status_sharing_duty),
    SimilarDuty(42, R.string.status_similar_duty),
    InDuty(43, R.string.status_in_duty),
    TrialAdventurer(44, R.string.status_trial_adventurer),
    FreeCompany(45, R.string.status_free_company),
    GrandCompany(46, R.string.status_grand_company),
    Online(47, R.string.status_online);

    companion object {
        private val map = values().associateBy(OnlineStatus::code)
        fun fromCode(code: Byte) = this.map[code]
    }
}
