package io.annaclemens.xivchat.model.message

import org.msgpack.core.MessagePack

data class RelayRegister(
    val authToken: String,
    val target: ByteArray,
) {
    fun encode(): ByteArray {
        val packer = MessagePack.newDefaultBufferPacker()
        packer.packArrayHeader(2)
        packer.packString(this.authToken)
        packer.packBinaryHeader(this.target.size)
        packer.addPayload(this.target)
        packer.close()

        return packer.toByteArray()
    }

    override fun equals(other: Any?): Boolean {
        if (this === other) {
            return true
        }

        @Suppress("ImplicitThis")
        if (this.javaClass != other?.javaClass) {
            return false
        }

        other as RelayRegister

        if (this.authToken != other.authToken) {
            return false
        }
        if (!this.target.contentEquals(other.target)) {
            return false
        }

        return true
    }

    override fun hashCode(): Int {
        var result = this.authToken.hashCode()
        result = 31 * result + this.target.contentHashCode()
        return result
    }
}

data class SuccessMessage(
    val success: Boolean,
    val info: String?,
) {
    companion object {
        fun read(input: ByteArray): SuccessMessage {
            val unpacker = MessagePack.newDefaultUnpacker(input)

            unpacker.unpackArrayHeader()

            val success = unpacker.unpackBoolean()
            val info = if (unpacker.tryUnpackNil()) {
                null
            } else {
                unpacker.unpackString()
            }

            return SuccessMessage(success, info)
        }
    }
}
