package io.annaclemens.xivchat.ui.about

import android.content.Intent
import android.net.Uri
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TableLayout
import android.widget.TableRow
import android.widget.TextView
import androidx.fragment.app.Fragment
import io.annaclemens.xivchat.R
import io.annaclemens.xivchat.databinding.FragmentAboutBinding

class AboutFragment : Fragment() {
    companion object {
        private const val KOFI_URL: String = "https://ko-fi.com/ascclemens"
        private const val PATREON_URL: String = "https://www.patreon.com/ascclemens"
        private const val LIBERAPAY_URL: String = "https://liberapay.com/ascclemens"
    }

    private fun openUrl(url: String) {
        val i = Intent(Intent.ACTION_VIEW)
        i.data = Uri.parse(url)
        this.startActivity(i)
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        val binding = FragmentAboutBinding.inflate(inflater)

        binding.kofiImage.setOnClickListener {
            this.openUrl(Companion.KOFI_URL)
        }
        binding.liberapayImage.setOnClickListener {
            this.openUrl(Companion.LIBERAPAY_URL)
        }
        binding.patreonImage.setOnClickListener {
            this.openUrl(Companion.PATREON_URL)
        }

        val translators = this.resources.getStringArray(R.array.translators)
        val translatorsLayout = TableLayout(this.context)
        for (translator in translators) {
            val split = translator.split('|', limit = 2)

            val langId = when (split[0]) {
                "en" -> R.string.lang_en
                "de" -> R.string.lang_de
                "fr" -> R.string.lang_fr
                "ja", "jp" -> R.string.lang_ja
                else -> 0
            }
            val langName = if (langId != 0) {
                this.getString(langId)
            } else {
                "Unknown"
            }

            val name = split[1]

            val row = TableRow(this.context)
            val layoutParams = TableRow.LayoutParams().apply {
                this.marginEnd = this@AboutFragment.dpToPx(8)
            }
            row.addView(TextView(this.context).apply { this.text = langName }, layoutParams)
            row.addView(TextView(this.context).apply { this.text = name })

            translatorsLayout.addView(row)
        }

        binding.aboutLayout.addView(translatorsLayout)

        return binding.root
    }

    private fun dpToPx(dp: Int): Int {
        return (this.resources.displayMetrics.scaledDensity * dp).toInt()
    }
}
