package io.annaclemens.xivchat.ui.friends

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.activity.viewModels
import androidx.fragment.app.Fragment
import androidx.recyclerview.widget.LinearLayoutManager
import io.annaclemens.xivchat.databinding.FragmentFriendListBinding
import io.annaclemens.xivchat.ui.servers.ServersViewModel
import io.annaclemens.xivchat.util.getApp

class FriendListFragment : Fragment() {
    private lateinit var viewAdapter: FriendListAdapter

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        val vm: FriendListViewModel by this.getApp().viewModels()
        val binding = FragmentFriendListBinding.inflate(inflater)

        if (vm.friends.value?.isEmpty() != false) {
            binding.friendListInstructions.visibility = View.VISIBLE
        }

        binding.friendListRefresh.setOnRefreshListener {
            val serversVm: ServersViewModel by this.getApp().viewModels()
            if (serversVm.connected.value != true) {
                binding.friendListRefresh.isRefreshing = false
                return@setOnRefreshListener
            }
            serversVm.connectionService.value?.requestFriendList()
        }

        this.viewAdapter = FriendListAdapter(this.requireContext())
        binding.friendList.apply {
            this.setHasFixedSize(true)
            this.layoutManager = LinearLayoutManager(this.context)
            this.adapter = this@FriendListFragment.viewAdapter
        }

        vm.friends.observe(this.viewLifecycleOwner) {
            binding.friendListInstructions.visibility = if (it == null) View.VISIBLE else View.GONE
            binding.friendListEmptyText.visibility = if (it != null && it.isEmpty()) View.VISIBLE else View.GONE
            binding.friendListRefresh.isRefreshing = false
            this.viewAdapter.setPlayerList(it.orEmpty())
            this.viewAdapter.notifyDataSetChanged()
        }

        return binding.root
    }
}
