package io.annaclemens.xivchat.ui.messages

import android.content.SharedPreferences
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.Fragment
import androidx.lifecycle.LiveData
import androidx.lifecycle.ViewModelProvider
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import io.annaclemens.xivchat.MainActivity
import io.annaclemens.xivchat.databinding.FragmentMessagesBinding
import io.annaclemens.xivchat.model.Tab
import io.annaclemens.xivchat.util.XivChatMarkdownPlugin
import io.annaclemens.xivchat.util.getApp
import io.noties.markwon.Markwon
import io.noties.markwon.ext.strikethrough.StrikethroughPlugin

class MessagesFragment : Fragment() {
    companion object {
        const val ARG_TAB_ID = "tabId"
        const val DIALOG_PHISHING = "phishing"
        const val DIALOG_CHANNEL = "channelChooser"
    }

    private lateinit var messagesViewModel: MessagesViewModel
    private var viewAdapter: MessagesAdapter? = null
    private lateinit var viewManager: RecyclerView.LayoutManager

    private lateinit var markdownModeListener: SharedPreferences.OnSharedPreferenceChangeListener
    private lateinit var tab: LiveData<Tab>

    override fun onResume() {
        super.onResume()

        this.viewAdapter?.catchUp()?.invokeOnCompletion {
            this.getApp().runOnUiThread {
                this.viewAdapter?.registerObserver()
            }
        }
    }

    override fun onPause() {
        super.onPause()

        this.viewAdapter?.unregisterObserver()
    }

    fun searchString(query: String?) {
        this.viewAdapter?.filter?.filterString = query
        this.viewAdapter?.filterExisting()
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        this.messagesViewModel = ViewModelProvider(this.requireActivity() as MainActivity).get(MessagesViewModel::class.java)
        val binding = FragmentMessagesBinding.inflate(inflater)

        val tabId = this.requireArguments().getInt(Companion.ARG_TAB_ID)
        this.tab = this.messagesViewModel.getTab(tabId)

        this.viewManager = LinearLayoutManager(this.context).apply {
            this.reverseLayout = true
        }
        val markdown = Markwon.builder(this.requireContext())
            .usePlugin(StrikethroughPlugin.create())
            .usePlugin(XivChatMarkdownPlugin())
            .build()

        this.tab.observe(this.viewLifecycleOwner) {
            if (it == null) {
                return@observe
            }

            this.viewAdapter = MessagesAdapter(this.getApp(), this, this.messagesViewModel, this.viewManager, markdown, it)
            binding.messages.apply {
                this.setHasFixedSize(false)
                this.layoutManager = this@MessagesFragment.viewManager
                this.adapter = this@MessagesFragment.viewAdapter
            }
        }

        // handle markdown mode
        this.markdownModeListener = object : SharedPreferences.OnSharedPreferenceChangeListener {
            override fun onSharedPreferenceChanged(sharedPreferences: SharedPreferences?, key: String?) {
                if (key != MainActivity.CONFIG_MARKDOWN_MODE) {
                    return
                }

                binding.messages.invalidate()
            }
        }
        this.getApp().preferences.registerOnSharedPreferenceChangeListener(this.markdownModeListener)

        return binding.root
    }

    override fun onDestroyView() {
        super.onDestroyView()

        this.getApp().preferences.unregisterOnSharedPreferenceChangeListener(this.markdownModeListener)
    }
}
