package io.annaclemens.xivchat.ui.messages

import android.app.Application
import androidx.lifecycle.AndroidViewModel
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import io.annaclemens.xivchat.Database
import io.annaclemens.xivchat.model.ChatType
import io.annaclemens.xivchat.model.Tab
import io.annaclemens.xivchat.model.TabRepository
import io.annaclemens.xivchat.model.message.ServerMessage
import io.annaclemens.xivchat.model.message.TextChunk
import kotlinx.coroutines.launch
import java.util.*
import kotlin.collections.ArrayList

class MessagesViewModel(application: Application) : AndroidViewModel(application) {
    private val tabRepository: TabRepository
    val allTabs: LiveData<List<Tab>>

    private val _messages = ArrayList<ServerMessage>()
    val messages: List<ServerMessage>
        get() {
            return this._messages
        }
    val lastAdded: MutableLiveData<Collection<ServerMessage>> = MutableLiveData(emptyList())

    init {
        val dao = Database.getDatabase(application).tabDao()
        this.tabRepository = TabRepository(dao)
        this.allTabs = this.tabRepository.allTabs
    }

    fun getTab(id: Int): LiveData<Tab> = this.tabRepository.get(id)

    fun updateTab(tab: Tab) = this.viewModelScope.launch {
        this@MessagesViewModel.tabRepository.update(tab)
    }

    fun insertTab(tab: Tab) = this.viewModelScope.launch {
        this@MessagesViewModel.tabRepository.insert(tab)
    }

    fun deleteTab(tab: Tab) = this.viewModelScope.launch {
        this@MessagesViewModel.tabRepository.delete(tab)
    }

    fun deleteTab(id: Int) = this.viewModelScope.launch {
        this@MessagesViewModel.tabRepository.delete(id)
    }

    fun addDefaultTabs() = this.viewModelScope.launch {
        this@MessagesViewModel.tabRepository.addDefaults()
    }

    fun addMessage(message: ServerMessage) {
        this._messages.add(message)
        this.lastAdded.value = listOf(message)
    }

    fun addMessages(messages: Collection<ServerMessage>) {
        this._messages.addAll(messages)
        this.lastAdded.value = messages
    }

    fun clearMessages() {
        this._messages.clear()
    }

    @OptIn(ExperimentalUnsignedTypes::class)
    fun addSystemMessage(content: String) {
        val bytes = content.toByteArray()
        val message = ServerMessage(
            Date(),
            ChatType.AppSystem.code.toUShort(),
            ByteArray(0),
            bytes,
            listOf(
                TextChunk(0xb38cffff.toUInt(), null, null, false, content),
            ),
        )
        this.addMessage(message)
    }

    val available = MutableLiveData(false)
}
