package io.annaclemens.xivchat.ui.servers

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.Fragment
import androidx.lifecycle.ViewModelProvider
import androidx.navigation.fragment.findNavController
import androidx.recyclerview.widget.ItemTouchHelper
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import io.annaclemens.xivchat.MainActivity
import io.annaclemens.xivchat.R
import io.annaclemens.xivchat.databinding.FragmentServersBinding
import io.annaclemens.xivchat.model.SavedServerAdapter
import io.annaclemens.xivchat.util.AlertDialogFragment

class ServersFragment : Fragment() {
    companion object {
        const val DIALOG_DELETE_CONFIRM = "delete_confirm"
    }

    private lateinit var serversViewModel: ServersViewModel

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        this.serversViewModel = ViewModelProvider(this.requireActivity() as MainActivity).get(ServersViewModel::class.java)
        val binding = FragmentServersBinding.inflate(inflater)

        binding.addServer.apply {
            this.inflate(R.menu.server_fab)

            this.setOnActionSelectedListener { actionItem ->
                when (actionItem.id) {
                    R.id.fab_manual -> {
                        val action = ServersFragmentDirections.actionToNewServer(0, null, null)
                        this@ServersFragment.findNavController().navigate(action)
                    }
                    R.id.fab_automatic -> {
                        val action = ServersFragmentDirections.actionToScannedServers()
                        this@ServersFragment.findNavController().navigate(action)
                    }
                }

                this.close()
                false
            }
        }

        val itemTouchHelperCallback = object : ItemTouchHelper.SimpleCallback(0, ItemTouchHelper.LEFT or ItemTouchHelper.RIGHT) {
            override fun onMove(recyclerView: RecyclerView, viewHolder: RecyclerView.ViewHolder, target: RecyclerView.ViewHolder): Boolean {
                return false
            }

            override fun onSwiped(viewHolder: RecyclerView.ViewHolder, direction: Int) {
                when (direction) {
                    ItemTouchHelper.LEFT -> {
                        val serverId = (viewHolder as SavedServerAdapter.ViewHolder).server.id
                        val dialog = AlertDialogFragment.newInstance(Companion.DIALOG_DELETE_CONFIRM, Bundle().apply {
                            this.putInt("serverId", serverId)
                        })
                        dialog.show(this@ServersFragment.childFragmentManager, "server_delete_dialog")
                    }
                    ItemTouchHelper.RIGHT -> {
                        val holder = viewHolder as SavedServerAdapter.ViewHolder
                        val action = ServersFragmentDirections.actionToNewServer(
                            relay = holder.server.host == null && holder.server.auth != null,
                            id = holder.server.id,
                            name = holder.server.name,
                            host = holder.server.host,
                            port = holder.server.port ?: 0,
                            auth = holder.server.auth,
                            target = holder.server.target,
                        )
                        this@ServersFragment.findNavController().navigate(action)
                    }
                }
            }
        }

        this.serversViewModel.connected.observe(this.viewLifecycleOwner) {
            binding.serverList.adapter?.notifyDataSetChanged()
        }

        binding.serverList.apply {
            this.adapter = SavedServerAdapter(this@ServersFragment.requireContext(), this@ServersFragment.serversViewModel)
            this.layoutManager = LinearLayoutManager(this@ServersFragment.requireContext())
            ItemTouchHelper(itemTouchHelperCallback).attachToRecyclerView(this)
        }

        this.serversViewModel.allServers.observe(this.viewLifecycleOwner) {
            if (it == null || it.isEmpty()) {
                binding.emptyMessage.visibility = View.VISIBLE
                binding.instructions.visibility = View.GONE
                return@observe
            }
            binding.emptyMessage.visibility = View.GONE
            binding.instructions.visibility = View.VISIBLE
            (binding.serverList.adapter as SavedServerAdapter).setServers(it)
        }

        return binding.root
    }
}
