package io.annaclemens.xivchat.ui.servers

import android.app.Application
import android.graphics.Bitmap
import androidx.lifecycle.AndroidViewModel
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import io.annaclemens.xivchat.ConnectionService
import io.annaclemens.xivchat.Database
import io.annaclemens.xivchat.model.SavedServer
import io.annaclemens.xivchat.model.SavedServerRepository
import io.annaclemens.xivchat.model.message.PlayerData
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch

class ServersViewModel(application: Application) : AndroidViewModel(application) {
    private val repository: SavedServerRepository
    val allServers: LiveData<List<SavedServer>>

    val connectionService: MutableLiveData<ConnectionService> = MutableLiveData<ConnectionService>()
    val connected = MutableLiveData<Boolean>().apply {
        this.value = false
    }
    var connectedServerId: Int = -1
    var lastConnectedServerId: Int = -1

    val playerData: MutableLiveData<PlayerData> = MutableLiveData()
    val playerPortrait: MutableLiveData<Bitmap> = MutableLiveData()

    val channel: MutableLiveData<String> = MutableLiveData()

    init {
        val dao = Database.getDatabase(application).serverDao()
        this.repository = SavedServerRepository(dao)
        this.allServers = this.repository.allServers
    }

    fun insert(server: SavedServer) = this.viewModelScope.launch(Dispatchers.IO) {
        this@ServersViewModel.repository.insert(server)
    }

    fun update(server: SavedServer) = this.viewModelScope.launch(Dispatchers.IO) {
        this@ServersViewModel.repository.update(server)
    }

    fun delete(server: SavedServer) = this.viewModelScope.launch(Dispatchers.IO) {
        this@ServersViewModel.repository.delete(server)
    }

    fun delete(id: Int) = this.viewModelScope.launch(Dispatchers.IO) {
        this@ServersViewModel.repository.delete(id)
    }
}
