package io.annaclemens.xivchat.ui.servers.scan

import kotlin.time.Duration
import kotlin.time.ExperimentalTime
import kotlin.time.TimeMark
import kotlin.time.TimeSource

@OptIn(ExperimentalUnsignedTypes::class, ExperimentalTime::class)
data class ScannedServer(
    val playerName: String,
    val address: String,
    val port: UShort,
    val publicKey: ByteArray,
) {
    private var timestamp: TimeMark = TimeSource.Monotonic.markNow()

    fun bumpTimestamp() {
        this.timestamp = TimeSource.Monotonic.markNow()
    }

    fun hasExpired() = this.timestamp.elapsedNow() >= Duration.seconds(10)

    override fun equals(other: Any?): Boolean {
        if (this === other) return true
        @Suppress("ImplicitThis")
        if (this.javaClass != other?.javaClass) return false

        other as ScannedServer

        if (this.address != other.address) return false
        if (this.port != other.port) return false
        if (!this.publicKey.contentEquals(other.publicKey)) return false

        return true
    }

    override fun hashCode(): Int {
        var result = this.address.hashCode()
        result = 31 * result + this.port.hashCode()
        result = 31 * result + this.publicKey.contentHashCode()
        return result
    }
}
