package io.annaclemens.xivchat.ui.servers.scan

import android.annotation.SuppressLint
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.TextView
import androidx.navigation.fragment.findNavController
import androidx.recyclerview.widget.RecyclerView
import io.annaclemens.xivchat.R
import io.annaclemens.xivchat.databinding.SavedServerBinding

class ScannedServerAdapter internal constructor(private val fragment: ScannedServersFragment) : RecyclerView.Adapter<ScannedServerAdapter.ViewHolder>() {
    private val inflater = LayoutInflater.from(this.fragment.context)
    private var servers = emptyList<ScannedServer>()

    inner class ViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {
        private val binding = SavedServerBinding.bind(itemView)

        lateinit var server: ScannedServer
        val name: TextView = this.binding.name
        val address: TextView = this.binding.address
        val connectedCheck: ImageView = this.binding.connectedCheck
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolder {
        val itemView = this.inflater.inflate(R.layout.saved_server, parent, false)
        return this.ViewHolder(itemView)
    }

    @OptIn(ExperimentalUnsignedTypes::class)
    override fun onBindViewHolder(holder: ViewHolder, position: Int) {
        val current = this.servers[position]

        holder.server = current
        holder.connectedCheck.visibility = View.GONE
        holder.name.text = current.playerName
        @SuppressLint("SetTextI18n")
        holder.address.text = "${current.address}:${current.port}"
        holder.itemView.setOnClickListener {
            val action = ScannedServersFragmentDirections.actionAddScannedServer(
                id = -1,
                name = current.playerName,
                host = current.address,
                port = current.port.toInt(),
            )
            this.fragment.findNavController().navigate(action)
        }
    }

    internal fun setServers(servers: List<ScannedServer>) {
        this.servers = servers
        this.notifyDataSetChanged()
    }

    override fun getItemCount() = this.servers.size
}
