package io.annaclemens.xivchat.ui.servers.scan

import android.content.ComponentName
import android.content.Context
import android.content.Intent
import android.content.ServiceConnection
import android.os.Bundle
import android.os.IBinder
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.activity.viewModels
import androidx.fragment.app.Fragment
import androidx.recyclerview.widget.LinearLayoutManager
import io.annaclemens.xivchat.databinding.FragmentScannedServersBinding
import io.annaclemens.xivchat.util.getApp

class ScannedServersFragment : Fragment() {
    private val serviceConnection = object : ServiceConnection {
        override fun onServiceConnected(name: ComponentName?, binder: IBinder?) {
            Log.d(this::class.qualifiedName, "onServiceConnected")
            if (binder !is ServerScannerService.LocalBinder) {
                Log.d(this::class.qualifiedName, "Binder is not right")
                return
            }

            val service = binder.getService()
            service.app = this@ScannedServersFragment.getApp()
            service.spawn()
            Log.d(this::class.qualifiedName, "Spawned service")
        }

        override fun onServiceDisconnected(name: ComponentName?) {
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        Log.d(this::class.qualifiedName, "Binding service")
        this.getApp().bindService(Intent(this.getApp(), ServerScannerService::class.java), this.serviceConnection, Context.BIND_AUTO_CREATE)
    }

    override fun onDestroy() {
        super.onDestroy()

        this.getApp().unbindService(this.serviceConnection)
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        val vm: ScannedServersViewModel by this.getApp().viewModels()
        val binding = FragmentScannedServersBinding.inflate(inflater)

        binding.serverList.apply {
            this.adapter = ScannedServerAdapter(this@ScannedServersFragment)
            this.layoutManager = LinearLayoutManager(this@ScannedServersFragment.context)
        }

        vm.scannedServers.observe(this.viewLifecycleOwner) {
            (binding.serverList.adapter as ScannedServerAdapter).setServers(it?.toList().orEmpty())

            if (it == null || it.isEmpty()) {
                binding.searchingLayout.visibility = View.VISIBLE
                binding.instructions.visibility = View.GONE
                return@observe
            }
            binding.searchingLayout.visibility = View.GONE
            binding.instructions.visibility = View.VISIBLE
        }

        return binding.root
    }
}
