package io.annaclemens.xivchat.util

import android.app.AlertDialog
import android.app.Dialog
import android.content.Context
import android.os.Bundle
import androidx.fragment.app.DialogFragment

class AlertDialogFragment : DialogFragment() {
    interface DialogHost {
        fun onDialogCreate(type: String, args: Bundle, builder: AlertDialog.Builder)
    }

    private lateinit var host: DialogHost

    private fun getType(): String {
        return this.requireArguments().getString(Companion.ARG_TYPE)!!
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)

        this.host = context as DialogHost
    }

    override fun onCreateDialog(savedInstanceState: Bundle?): Dialog {
        val builder = AlertDialog.Builder(this.activity)
        this.host.onDialogCreate(this.getType(), this.requireArguments(), builder)
        return builder.create()
    }

    companion object {
        private const val ARG_TYPE = "type"

        fun newInstance(type: String, args: Bundle = Bundle()): AlertDialogFragment {
            return AlertDialogFragment().apply {
                this.arguments = args.apply {
                    this.putString(this@Companion.ARG_TYPE, type)
                }
            }
        }
    }
}
