package io.annaclemens.xivchat.util

import com.goterl.lazycode.lazysodium.LazySodiumAndroid
import com.goterl.lazycode.lazysodium.interfaces.KeyExchange
import com.goterl.lazycode.lazysodium.utils.KeyPair
import com.goterl.lazycode.lazysodium.utils.SessionPair
import io.ktor.utils.io.*

class AppKeyExchange {
    companion object {
        private fun clientSessionKeys(sodium: LazySodiumAndroid, client: KeyPair, serverPublic: ByteArray): SessionPair {
            val kx = sodium as KeyExchange.Native

            val rx = ByteArray(KeyExchange.SESSIONKEYBYTES)
            val tx = ByteArray(KeyExchange.SESSIONKEYBYTES)
            kx.cryptoKxClientSessionKeys(rx, tx, client.publicKey.asBytes, client.secretKey.asBytes, serverPublic)

            return SessionPair(rx, tx)
        }

        suspend fun clientHandshake(sodium: LazySodiumAndroid, client: KeyPair, input: ByteReadChannel, output: ByteWriteChannel): HandshakeInfo {
            // send our public key
            output.writeFully(client.publicKey.asBytes)

            // get the server's public key
            val serverPublic = ByteArray(KeyExchange.PUBLICKEYBYTES)
            input.readFully(serverPublic)

            // compute shared keys
            val pair = this.clientSessionKeys(sodium, client, serverPublic)

            return HandshakeInfo(serverPublic, pair)
        }
    }

    data class HandshakeInfo(val remotePublicKey: ByteArray, val keys: SessionPair) {
        override fun equals(other: Any?): Boolean {
            if (this === other) return true
            @Suppress("ImplicitThis")
            if (this.javaClass != other?.javaClass) {
                return false
            }

            other as HandshakeInfo

            if (!this.remotePublicKey.contentEquals(other.remotePublicKey) || this.keys != other.keys) {
                return false
            }

            return true
        }

        override fun hashCode(): Int {
            var result = this.remotePublicKey.contentHashCode()
            result = 31 * result + this.keys.hashCode()
            return result
        }
    }
}

