package io.annaclemens.xivchat.util

import android.app.AlertDialog
import android.app.Dialog
import android.content.Context
import android.os.Bundle
import android.widget.EditText
import android.widget.LinearLayout
import androidx.fragment.app.DialogFragment

class EditTextDialog : DialogFragment() {
    interface DialogHost {
        fun getEditContext(): Context
        fun onCreateTextView(type: String, args: Bundle, edit: EditText)
        fun onCreateEditTextDialog(type: String, args: Bundle, builder: AlertDialog.Builder, edit: EditText)
    }

    private lateinit var host: DialogHost

    private fun getType(): String {
        return this.requireArguments().getString(Companion.ARG_TYPE)!!
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)

        this.host = context as DialogHost
    }

    override fun onCreateDialog(savedInstanceState: Bundle?): Dialog {
        val text = EditText(this.host.getEditContext()).apply {
            this@EditTextDialog.host.onCreateTextView(this@EditTextDialog.getType(), this@EditTextDialog.requireArguments(), this)
            this.setSingleLine()
            this.layoutParams = LinearLayout.LayoutParams(
                LinearLayout.LayoutParams.MATCH_PARENT,
                LinearLayout.LayoutParams.WRAP_CONTENT,
            ).apply {
                this.marginStart = (20 * this@EditTextDialog.host.getEditContext().resources.displayMetrics.density).toInt()
                this.marginEnd = this.marginStart
            }
        }
        val linear = LinearLayout(this.host.getEditContext()).apply {
            this.addView(text)
        }
        val builder = AlertDialog.Builder(this.activity)
        this.host.onCreateEditTextDialog(this@EditTextDialog.getType(), this@EditTextDialog.requireArguments(), builder, text)
        builder.setView(linear)
        return builder.create()
    }

    companion object {
        private const val ARG_TYPE = "type"

        fun newInstance(type: String, args: Bundle = Bundle()): EditTextDialog {
            return EditTextDialog().apply {
                this.arguments = args.apply {
                    this.putString(this@Companion.ARG_TYPE, type)
                }
            }
        }
    }
}
