package io.annaclemens.xivchat.util

import androidx.fragment.app.Fragment
import io.annaclemens.xivchat.MainActivity

fun ByteArray.toHexString(upper: Boolean = false, separator: String = ""): String {
    val format = if (upper) "%02X" else "%02x"

    return this
        .map { it.toInt() and 0xFF }
        .joinToString(separator) {
            format.format(it)
        }
}

fun ByteArray.toColours(): List<Long> {
    return this
        .asIterable()
        .map { it.toLong() and 0xFF }
        .chunked(3) {
            val a = it[0]
            val b = it.getOrElse(1) { 0 }
            val c = it.getOrElse(2) { 0 }
            0xFF000000 or (a shl 16) or (b shl 8) or c
        }
        .toList()
}

private const val UPPER_HEX_CHARS = "0123456789ABCDEF"
private const val LOWER_HEX_CHARS = "0123456789abcdef"

fun String.hexStringToByteArray(): ByteArray? {
    val result = ByteArray(this.length / 2)

    for (i in this.indices step 2) {
        val firstIndex = UPPER_HEX_CHARS.indexOf(this[i]).takeIf { it >= 0 } ?: LOWER_HEX_CHARS.indexOf(this[i])
        val secondIndex = UPPER_HEX_CHARS.indexOf(this[i + 1]).takeIf { it >= 0 } ?: LOWER_HEX_CHARS.indexOf(this[i + 1])

        if (firstIndex == -1 || secondIndex == -1) {
            return null
        }

        val octet = firstIndex.shl(4).or(secondIndex)
        result[i.shr(1)] = octet.toByte()
    }

    return result
}

fun Fragment.getApp(): MainActivity {
    return this.requireActivity() as MainActivity
}
