package io.annaclemens.xivchat.util

import android.content.Context
import android.graphics.Rect
import android.graphics.Typeface
import android.text.SpannableStringBuilder
import android.text.Spanned
import android.text.style.ForegroundColorSpan
import android.text.style.StyleSpan
import androidx.core.content.ContextCompat
import androidx.core.text.inSpans
import io.annaclemens.xivchat.R
import io.annaclemens.xivchat.model.message.Chunk
import io.annaclemens.xivchat.model.message.IconChunk
import io.annaclemens.xivchat.model.message.TextChunk
import io.noties.markwon.Markwon

@ExperimentalUnsignedTypes
class SeString {
    companion object {
        fun chunksToSpans(chunks: List<Chunk>, markdown: Markwon?, context: Context?): Spanned {
            val builder = SpannableStringBuilder()

            for (chunk in chunks) {
                when (chunk) {
                    is TextChunk -> builder.inSpans(*this.calculateSpans(chunk.italic, chunk.argb())) {
                        this.append(chunk.content, markdown)
                    }
                    is IconChunk -> {
                        if (context == null) {
                            continue
                        }
                        val bounds = iconBounds(chunk.index.toInt()) ?: continue
                        val fontIcon = ContextCompat.getDrawable(context, R.drawable.fonticon_ps4) ?: continue
                        val cropped = CroppedDrawable(fontIcon, bounds).apply {
                            val width = (bounds.width() * context.resources.displayMetrics.density / 2).toInt()
                            val height = (bounds.height() * context.resources.displayMetrics.density / 2).toInt()
                            this.bounds = Rect(0, 0, width, height)
                        }
                        val imageSpan = CentredImageSpan(cropped)
                        builder.inSpans(imageSpan) {
                            this.append(' ')
                        }
                    }
                }
            }

            return builder
        }

        private fun calculateSpans(italic: Boolean, colour: Long?): Array<out Any> {
            val spans = ArrayList<Any>()

            if (italic) {
                spans.add(StyleSpan(Typeface.ITALIC))
            }

            if (colour != null) {
                spans.add(ForegroundColorSpan(colour.toInt()))
            }

            return spans.toArray()
        }

        private fun SpannableStringBuilder.append(text: String, markdown: Markwon?) {
            if (markdown == null) {
                // just normal text
                this.append(text)
            } else {
                val spacePrefixIdx = text.indexOfFirst { !it.isWhitespace() }
                val spaceSuffixIdx = text.indexOfLast { !it.isWhitespace() }
                val spacePrefix = if (spacePrefixIdx > -1) text.subSequence(0, spacePrefixIdx) else null
                val spaceSuffix = if (spaceSuffixIdx > -1) text.subSequence(spaceSuffixIdx + 1, text.length) else null

                // append the spaces at the start that will be trimmed
                if (spacePrefix != null) {
                    this.append(spacePrefix)
                }
                // process as markdown
                this.append(markdown.toMarkdown(text))
                // append the spaces at the end that will be trimmed
                if (spaceSuffix != null) {
                    this.append(spaceSuffix)
                }
            }
        }
    }
}
