package io.annaclemens.xivchat.util

import com.goterl.lazycode.lazysodium.LazySodiumAndroid
import com.goterl.lazycode.lazysodium.interfaces.SecretBox
import com.goterl.lazycode.lazysodium.utils.Key
import io.ktor.utils.io.*
import java.nio.ByteBuffer
import java.nio.ByteOrder

class SecretMessage {
    companion object {
        suspend fun readSecretMessage(sodium: LazySodiumAndroid, stream: ByteReadChannel, key: Key): ByteArray {
            val headerBytes = ByteArray(4 + 24)
            stream.readFully(headerBytes)
            // FIXME: little endian should be big endian
            val header = ByteBuffer.wrap(headerBytes).apply { this.order(ByteOrder.LITTLE_ENDIAN) }

            val length = header.int
            val nonce = ByteArray(header.remaining())
            header.get(nonce)

            val ciphertext = ByteArray(length)
            stream.readFully(ciphertext)
            val plaintext = ByteArray(ciphertext.size - SecretBox.MACBYTES)

            val secretBox = sodium as SecretBox.Native
            secretBox.cryptoSecretBoxOpenEasy(
                plaintext,
                ciphertext,
                ciphertext.size.toLong(),
                nonce,
                key.asBytes
            )

            return plaintext
        }

        suspend fun writeSecretMessage(sodium: LazySodiumAndroid, stream: ByteWriteChannel, key: Key, msg: ByteArray) {
            val secretBox = sodium as SecretBox.Native

            val nonce = sodium.nonce(SecretBox.NONCEBYTES)
            val ciphertext = ByteArray(SecretBox.MACBYTES + msg.size)
            secretBox.cryptoSecretBoxEasy(ciphertext, msg, msg.size.toLong(), nonce, key.asBytes)
            val len = ciphertext.size

            // FIXME: little endian should be big endian
            val buffer = ByteBuffer.allocate(nonce.size + ciphertext.size + 4).apply { this.order(ByteOrder.LITTLE_ENDIAN) }
            buffer.putInt(len)
            buffer.put(nonce)
            buffer.put(ciphertext)
            buffer.rewind()

            stream.writeFully(buffer.array())
        }
    }
}
