package io.annaclemens.xivchat.util

import android.app.Activity
import android.os.Bundle
import android.os.ResultReceiver
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.DialogFragment
import io.annaclemens.xivchat.R
import io.annaclemens.xivchat.databinding.TrustDialogBinding

class TrustDialog : DialogFragment() {
    companion object {
        const val TAG = "trustDialog"

        private const val SERVER_PUBLIC = "serverPublic"
        private const val CLIENT_PUBLIC = "clientPublic"
        private const val RECEIVER = "receiver"

        fun newInstance(receiver: ResultReceiver, serverPublic: ByteArray, clientPublic: ByteArray): TrustDialog {
            val args = Bundle().apply {
                this.putParcelable(this@Companion.RECEIVER, receiver)
                this.putByteArray(this@Companion.SERVER_PUBLIC, serverPublic)
                this.putByteArray(this@Companion.CLIENT_PUBLIC, clientPublic)
            }
            return TrustDialog().apply {
                this.arguments = args
            }
        }

    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        return inflater.inflate(R.layout.trust_dialog, container)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        val receiver = this.arguments?.getParcelable<ResultReceiver>(Companion.RECEIVER) ?: throw IllegalArgumentException("receiver was null or not set")
        val serverPublic = this.arguments?.getByteArray(Companion.SERVER_PUBLIC) ?: ByteArray(0)
        val clientPublic = this.arguments?.getByteArray(Companion.CLIENT_PUBLIC) ?: ByteArray(0)

        val binding = TrustDialogBinding.bind(view)

        binding.trustServerPublicKey.text = this@TrustDialog.getString(R.string.connect_trust_server).format(serverPublic.toHexString(upper = true))
        binding.trustServerPublicKeyColours.generateDrawables(serverPublic)

        binding.trustClientPublicKey.text = this@TrustDialog.getString(R.string.connect_trust_client).format(clientPublic.toHexString(upper = true))
        binding.trustClientPublicKeyColours.generateDrawables(clientPublic)

        binding.trustButtonNo.setOnClickListener {
            receiver.send(Activity.RESULT_CANCELED, null)
            this.dismiss()
        }

        binding.trustButtonYes.setOnClickListener {
            receiver.send(Activity.RESULT_OK, null)
            this.dismiss()
        }
    }
}
