package io.annaclemens.xivchat.util

import android.graphics.Typeface
import io.noties.markwon.AbstractMarkwonPlugin
import io.noties.markwon.MarkwonSpansFactory
import io.noties.markwon.core.spans.CustomTypefaceSpan
import org.commonmark.ext.autolink.AutolinkExtension
import org.commonmark.node.AbstractVisitor
import org.commonmark.node.Emphasis
import org.commonmark.node.Image
import org.commonmark.node.Text
import org.commonmark.parser.Parser


class XivChatMarkdownPlugin : AbstractMarkwonPlugin() {
    override fun configureParser(builder: Parser.Builder) {
        builder
            // auto-link bare links
            .extensions(listOf(
                AutolinkExtension.create(),
            ))
            // don't parse any block types
            .enabledBlockTypes(emptySet())
            // disable images
            .postProcessor {
                it.accept(object : AbstractVisitor() {
                    override fun visit(image: Image?) {
                        if (image == null) {
                            return
                        }
                        // FIXME: title is always null for some reason
                        val text = "![${image.title ?: ""}](${image.destination ?: ""})"
                        image.insertAfter(Text(text))
                        image.unlink()
                    }
                })
                it
            }
    }

    override fun configureSpansFactory(builder: MarkwonSpansFactory.Builder) {
        // make italics match normal system italics
        builder.setFactory(Emphasis::class.java) { _, _ ->
            CustomTypefaceSpan(Typeface.create(Typeface.DEFAULT, Typeface.ITALIC))
        }
    }
}